# Documentation string. I wonder what ever refers to it? 

__doc__ = """KZMailman product module.

See README.txt for more details."""

__version__ = '0.1.1'  

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB
from Globals import InitializeClass

import OFS.SimpleItem
import Acquisition
import AccessControl.Role

import os, sys, httplib, urllib, urlparse
import popen2
import string
import tempfile

from AccessControl import ClassSecurityInfo

class KMMException(Exception) : pass

manage_addKZMailmanForm = HTMLFile('KZMailmanAdd', globals()) 

def manage_addKZMailman(self, id, title='', addAliases=None, aliasesPath='', mailmanPath='', mailmanURL='', sitePassword='', aliasUpdateCmd='', REQUEST=None):
    """Add a KZMailman instance to a folder."""
    if addAliases != None :
       addAliases = 1
    else :
       addAliases = 0
    self._setObject(id, KZMailman(id, title, addAliases, aliasesPath, mailmanPath, mailmanURL, sitePassword, aliasUpdateCmd))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class KZMailman(
    OFS.SimpleItem.Item,   # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """KZMailman object.  This object performs the integration between
       Zope and Mailman. 
    """
        
    meta_type = 'KZMailman' 
    security = ClassSecurityInfo()

    manage_options = ( # what management options are there?
	{'label': 'Edit',       'action': 'manage_main'},
	{'label': 'View',       'action': ''}, # defaults to index_html
        {'label': 'Manage List Members', 'action':'manage_list_members'},
        {'label': 'Manage List Options', 'action':'manage_list_options'},
	{'label': 'Security',   'action': 'manage_access'},
        ) 

    __ac_permissions__=( # what permissions make sense for us? 
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions',      ('manage_access',)           ),
	('Admin KZMailman'     ,     ('manage_edit', 'createMailList', 'deleteMailList', 'getLists') ),
	('View KZMailman',            ('',)                        ),
        ('Add Mailman List Member', ('addMember',)           ),
        ('Remove Mailman List Member', ('removeMember',)        ),
        ('Get Mailman List Members', ('getMembers') ),
        ('Change Mailman List Options', ('changeListOptions',)       )
	)

    def __init__(self, id, title='', addAliases=0, aliasesPath='', mailmanPath='', mailmanURL='', execPath='', sitePassword='', aliasUpdateCmd=''): 
        """initialise a new instance of KZMailman"""
        self.id = id
        self.title = title
        self.addAliases = addAliases
        self.aliasesPath = aliasesPath
        self.mailmanPath = mailmanPath
        self.mailmanURL = mailmanURL
        if self.mailmanPath[-1] == '/' :           # remove trailing '/'
          self.mailmanPath = self.mailmanPath[0:-1]
        if self.mailmanURL[-1] == '/' :            # remove trailing '/'
          self.mailmanURL = self.mailmanURL[0:-1] 
        self.execPath = execPath
        self.sitePassword = sitePassword
	self.aliasUpdateCmd = aliasUpdateCmd

    index_html = HTMLFile('index', globals()) # View Interface
    
    manage_main = HTMLFile('KZMailmanEdit', globals()) # Management Interface
    
    def manage_edit(self, title, REQUEST=None): 
        """does this really need a doc string?"""
        self.title = title
        if REQUEST is not None:
            if REQUEST.get('addAliases') != None :
               self.addAliases = 1
            else :
               self.addAliases = 0
            self.mailmanPath = REQUEST.get('mailmanPath')
            if self.mailmanPath[-1] == '/' :           # remove trailing '/'
               self.mailmanPath = self.mailmanPath[0:-1]
            self.mailmanURL = REQUEST.get('mailmanURL')
            if self.mailmanURL[-1] == '/' :            # remove trailing '/'
               self.mailmanURL = self.mailmanURL[0:-1]
            self.aliasesPath = REQUEST.get('aliasesPath')
            self.sitePassword = REQUEST.get('sitePassword')
            self.aliasUpdateCmd = REQUEST.get('aliasUpdateCmd')
            self.execPath = REQUEST.get('execPath')
            return self.manage_main(self, REQUEST, manage_tabs_message="Properties updated.")  

    manage_list_members = HTMLFile('KZMailmanListMembers', globals()) # Management Interface
    manage_list_options = HTMLFile('KZMailmanListOptions', globals()) # Management Interface


##########################################################################################
# 
# createMailList : This function programmatically creates a mailman mailist list and
#                  changes the privacy and general options of the list to match what
#                  is needed by the system.  It also adds the aliases to the sendmail
#                  aliases file.
#
# Parameters:
#                  listName:    The name of the list to create
#                  adminEmail:  The email of the list administrator
#                  password:    The administrative password for this list
#
##########################################################################################

    security.declareProtected('Admin KZMailman', 'createMailList')

    def createMailList(self, listName, adminEmail, password) :
       ''' Function creates maillist, aliases for mailing list and
           adds them to the aliases file.  The second task it serves is to
           post information to the maillist cgi-bin modules '''

       nlCommand = "%s/bin/newlist -q %s %s %s" % \
                   (self.mailmanPath, listName, adminEmail, password)

       ouptut, errors = self._runCommand(nlCommand)

       if errors:
         raise KMMException, errors

       else :
         # Create aliases if required
         if self.addAliases:
           self._createAliases(listName)



##########################################################################################
# 
# deleteMailList  : This function deletes a given Mailman mailing list.
#                  
#                  
#
# Parameters:
#             listName:         The name of the mailman to which a user needs to be added.
#             removeArchives:   A flag indicating whether or not the email list archives
#                               should also be deleted.
#
##########################################################################################

    security.declareProtected('Admin KZMailman', 'deleteMailList')

    def deleteMailList(self, listName, removeArchives=0) :

       if removeArchives:
          nlCommand = "%s/bin/rmlist -a %s" % (self.mailmanPath, listName)
       else :
          nlCommand = "%s/bin/rmlist %s" % (self.mailmanPath, listName)

       output, errors = self._runCommand(nlCommand)

       if errors:
         raise KMMException, errors

       else :
         # Remove aliases if necessary
         if self.addAliases:
           self._removeAliases(listName)


##########################################################################################
# 
# addMember  : This function adds a user to a given Mailman mailing list.
#                  
#                  
#
# Parameters:
#             listName:    The name of the mailman to which a user needs to be added.
#             password:    The administrative password for the list.
#             userEmail:   The email address that is to be added to the list.
#             sendWelcome: Send a welcome email?  (default no)
#
##########################################################################################

    security.declareProtected('Add Mailman List Member', 'addMember')

    def addMember(self, listName, password, userEmail, sendWelcome=None) :
      ''' Function creates maillist, aliases for mailing list and
          adds them to the aliases file.  It then calls
          newaliases.  The second task it serves is to
          post information to the maillist cgi-bin modules '''

      tmpFile = tempfile.mktemp()
      inFile = open(tmpFile, 'w')
      inFile.write(userEmail)
      inFile.close()

      if sendWelcome:
        nlCommand = "%s/bin/add_members -r %s -w y %s" % (self.mailmanPath, tmpFile, listName)
      else :
        nlCommand = "%s/bin/add_members -r %s -w n %s" % (self.mailmanPath, tmpFile, listName)
        
      output, errors = self._runCommand(nlCommand)
      os.remove(tmpFile)

      if errors:
         raise KMMException, errors

##########################################################################################
# 
# addMemberWithPassword  : This function adds a user to a given Mailman mailing list.
#                  
#                  
#
# Parameters:
#             listName:    The name of the mailman to which a user needs to be added.
#             password:    The administrative password for the list.
#             userEmail:   The email address that is to be added to the list.
#             userPassword:Initial password for the member
#             sendWelcome: Send a welcome email?  (default no)
#
##########################################################################################

    security.declareProtected('Add Mailman List Member', 'addMember')

    def addMemberWithPassword(self, listName, password, userEmail, userPassword=None, sendWelcome=None) :
      ''' Function adds new member to mailing list with
          and inital password '''

      nlCommand = '%s/bin/add_member -r %s' % (self.mailmanPath, userEmail)

      if sendWelcome:
        nlCommand += " -w y"
      else :
        nlCommand += " -w n"

      if userPassword:
        nlCommand += " -p %s" % userPassword

      nlCommand += " %s" % listName
      
      output, errors = self._runCommand(nlCommand)

      if errors:
         raise KMMException, errors


##########################################################################################
# 
# removeMember  : This function removes a user from a given Mailman mailing list.
#                  
#                  
#
# Parameters:
#             listName:    The name of the mailman to which a user needs to be added.
#             userEmail:   The email address that is to be added to the list.
#
##########################################################################################

    security.declareProtected('Remove Mailman List Member', 'removeMember')

    def removeMember(self, listName, userEmail) :
      nlCommand = "%s/bin/remove_members %s %s" % \
                  (self.mailmanPath, listName, userEmail)

      output, errors = self._runCommand(nlCommand)

      if errors:
         raise KMMException, errors

##########################################################################################
# 
# getMembers  : This function displays users' emails of a given list
#                  
#                  
#
# Parameters:
#             listName:    The name of the mailman to which a user needs to be added.
#             regular:     Limit to regular subscriptions
#             digest:      Limit to digest subscriptions
#
##########################################################################################

    security.declareProtected('Get Mailman List Members', 'getMembers')

    def getMembers(self, listName, regular=0, digest=0) :

       if regular:
         rFlag = '-r'
       else :
         rFlag = ''

       if digest:
         dFlag = '-d'
       else :
         dFlag = ''

       nlCommand = "%s/bin/list_members %s %s %s" % \
                   (self.mailmanPath, rFlag, dFlag, listName)

       output, errors = self._runCommand(nlCommand)

       if errors:
         raise KMMException, errors

       if output:
         memberList = output.split('\n')

       else :
         memberList = []

       # Strip off last element which is always a blank string
       return memberList[:-1]

##########################################################################################
# 
# getLists  : This function gets list of Mailman lists
#                  
##########################################################################################

    security.declareProtected('Admin KZMailman', 'getLists')

    def getLists(self) :

       nlCommand = "%s/bin/list_lists -b" % (self.mailmanPath)

       output, errors = self._runCommand(nlCommand)

       if errors:
         raise KMMException, errors

       if output:
         listList = output.split('\n')

       else:
         listList = []

       # Trailing \n causes empty last line
       return listList[:-1]

##########################################################################################
# 
# deleteEmails: 
#                  
##########################################################################################

    security.declarePrivate('deleteEmails')

    def deleteEmails(self, REQUEST=None) :
       keys = REQUEST.keys()
       delKeys = [key for key in keys if key[0:7] == 'delete_']
       userEmails = [key.split('_')[1] for key in delKeys]
       listName = REQUEST.get('listName')

       for user in userEmails :
          self.removeMember(listName, user)
       return

    security.declareProtected('Admin KZMailman', 'getListOptions')

##########################################################################################
# 
# _createAliases : This function creates the necessary aliases in the mail programs aliases
#                 file.
#
# Parameters:
#                   listName:    The name of the mailing list to alter
#
##########################################################################################

    security.declarePrivate('_createAliases')

    def _createAliases(self, listName) :

       try:
         fAliases = open(self.aliasesPath,'r')
         aliasContent = fAliases.read()
         fAliases.close()
       except IOError:
         raise KMMException, '%s not found' % self.aliasesPath

       header = '##Start %s\n' % listName
       footer = '##End %s\n' % listName

       if (header not in aliasContent and footer not in aliasContent):
         
         newLines = []
         suffixesAndCommands = {	'':		'post',
    			'-admin':	'admin',
				'-bounces':	'bounces',
				'-confirm':	'confirm',
				'-join':	'join',
				'-leave':	'leave',
				'-owner':	'owner',
				'-request':	'request',
				'-subscribe':	'subscribe',
				'-unsubscribe':	'unsubscribe'}

         header = '##Start %s\n' % listName
         footer = '##End %s\n' % listName
       
         newLines.append('##Start %s\n' % listName)

         for suffix,command in suffixesAndCommands.items():
           newLines.append('%s%s: "|%s %s %s"\n' % \
             (listName, suffix, self.execPath or (self.mailmanPath+'/mail/mailman'), \
              command, listName) )

         newLines.append('##End %s\n' % listName)

         fAliases = open(self.aliasesPath, 'a')
         fAliases.writelines(newLines)
         fAliases.close()

         # Update aliases database file
         if (self.aliasUpdateCmd):

           nlCommand = self.aliasUpdateCmd
           ouptut, errors = self._runCommand(nlCommand)

           if errors:
             raise KMMException, errors

##########################################################################################
# 
# _removeAliases : This function creates the necessary aliases in the mail programs aliases
#                 file.
#
# Parameters:
#                   listName:    The name of the mailing list to alter
#
##########################################################################################

    def _removeAliases(self, listName) :

       header = '##Start %s\n' % listName
       footer = '##End %s\n' % listName

       try:
         fAliases = open(self.aliasesPath, 'r')
         aliasContent = fAliases.readlines()
         fAliases.close()

       except IOError, error:
         raise KMMException, error

       try:
         startBlock = aliasContent.index(header)
         endBlock = aliasContent.index(footer)
         del aliasContent[startBlock:endBlock+1]
         fAliases = open(self.aliasesPath,'w')
         fAliases.writelines(aliasContent)
         fAliases.close()

       except ValueError:
         raise KMMException, '%s not found in alias file' % listName
       
       except IOError:
         raise KMMException, '%s not found' % self.aliasesPath

       # Update aliases database file if necessary
       if self.aliasUpdateCmd:

         nlCommand = self.aliasUpdateCmd
         ouptut, errors = self._runCommand(nlCommand)

         if errors:
           raise KMMException, errors

##########################################################################################
# 
# _runCommand: 
#                  
##########################################################################################

    security.declarePrivate('_runCommand')

    def _runCommand(self, nlcommand) :
       outf, inf, errf = popen2.popen3(nlcommand)
       output = outf.read()
       errors = errf.read()
       outf.close()
       inf.close()
       errf.close()
       return output, errors


InitializeClass(KZMailman)
